(* runtime configuration *)
(* solyga@gmx.de, 2024-06-28 *)
IMPLEMENTATION MODULE Config;

IMPORT Str, IO, FIO, Lib2, Lib3, Args, SO;

CONST
  prg = 'par';
  ver = 32;
  ext = '2024-06-29';


PROCEDURE WrInfo( VAR b: SO.TBuffer );
BEGIN
  SO.WrStr( b, prg );
  SO.WrStr( b, ' v' );
  SO.WrCard( b, ver, 0 );
  SO.WrStr( b, ' (' );
  SO.WrStr( b, ext );
  SO.WrStr( b, '): Quake PAK archiver.' );
END WrInfo;

PROCEDURE WrAuth( VAR b: SO.TBuffer );
BEGIN
  SO.WrStr( b, 'Author: Steffen Solyga <solyga@gmx.de>' );
END WrAuth;


PROCEDURE ParseArgs();

  PROCEDURE Help();
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.Clear( b ); SO.WrStr( b, "(* " ); WrInfo( b ); SO.WrStr( b, " *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "(* " ); WrAuth( b ); SO.WrStr( b, " *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "syntax = '" ); SO.WrStr( b, prg ); SO.WrStr( b, "' { option } [ action { option } [ pak ] ] ." );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "option = '-h'     (* show this info and exit *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "       | '-d'     (* show debug messages *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "       | '-v'     (* be verbose *) ." );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "action = 'l'      (* list archive *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "       | 't'      (* table archive (extended list) *) ." );
    IO.WrStr( s ); IO.WrLn();
    Lib3.SetReturnCode( 0 );
    HALT;
  END Help;

  PROCEDURE Error( t-: ARRAY OF CHAR );
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.WrStr( b, prg );
    SO.WrStr( b, '.Config.ParseArgs.Error: ' );
    SO.WrStr( b, t );
    IF Args.len > 0 THEN
      SO.WrStr( b, ": '" );
      SO.WrStr( b, Args.lit );
      SO.WrStr( b, "'" );
    END;
    SO.WrLn( b );
    SO.WrStr( b, "Try '" );
    SO.WrStr( b, prg );
    SO.WrStr( b, " -h' for help." );
    Lib3.FatalError( s );
  END Error;

  PROCEDURE GetCard( VAR x: LONGCARD );
  VAR
    ok: BOOLEAN;
  BEGIN
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok THEN Error( 'Invalid number' ) END;
  END GetCard;

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF    Str.Compare( Args.lit, '?' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'h' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'd' ) = 0 THEN
        data.dbg := TRUE;
      ELSIF Str.Compare( Args.lit, 'v' ) = 0 THEN
        data.vrb := TRUE;
      ELSE
        Error( 'Invalid option' );
      END;
      Args.Read();
    END;
  END Options;

  PROCEDURE Action();
  BEGIN
    IF Args.sym # Args.symArg THEN Error( 'Action expected' ) END;
    CASE CAP( Args.lit[0] ) OF
    | 'L': data.act := actList;
    | 'T': data.act := actTable;
    ELSE
      Error( 'Invalid action' );
    END;
    Args.Read();
  END Action;

BEGIN
  Args.Reset();
  Args.Read();
  Options();
  IF Args.sym = Args.symArg THEN
    Action();
    Options();
    IF Args.sym = Args.symArg THEN
      Str.Copy( data.arc, Args.lit );
      Args.Read();
      Options();
    END;
  END;
  IF Args.sym # Args.symEOA THEN Error( 'Unexpected argument' ) END;
END ParseArgs;

PROCEDURE InitDir();
VAR
  drv: SHORTCARD;
BEGIN
  drv := FIO.GetDrive();
  FIO.GetDir( drv, data.dir );
  Str.Prepend( data.dir, ':' );
  Str.Prepend( data.dir, CHR( ORD( 'A' ) + CARDINAL(drv-1) ) );
END InitDir;

PROCEDURE Show();
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;

  PROCEDURE Out();
  BEGIN
    (* must be 2x IO (not FIO) for redirected stream beeing correct *)
    IO.WrStr( s );
    IO.WrLn();
  END Out;

  PROCEDURE ShowHdr();
  BEGIN
    SO.Clear( b ); WrInfo( b ); Out();
    SO.Clear( b ); WrAuth( b ); Out();
    SO.Clear( b ); SO.WrStr( b, 'Config.data:' ); Out();
  END ShowHdr;

  PROCEDURE Begin( n-: ARRAY OF CHAR );
  CONST w = 3;
  BEGIN
    SO.Clear( b ); SO.WrStr( b, '  ' ); SO.WrStrAdj( b, n, -w ); SO.WrStr( b, ' : ' );
  END Begin;

  PROCEDURE ShowStr( n-: ARRAY OF CHAR; v-: ARRAY OF CHAR );
  BEGIN
    Begin( n ); SO.WrChar( b, "'" ); SO.WrStr( b, v ); SO.WrChar( b, "'" ); Out();
  END ShowStr;

  PROCEDURE ShowCard( n-: ARRAY OF CHAR; v: CARDINAL );
  CONST w = 1;
  BEGIN
    Begin( n ); SO.WrLngCard( b, v, w ); Out();
  END ShowCard;

BEGIN
  SO.Init( b, s );
  ShowHdr();
  ShowStr(  'prg', data.prg );
  ShowCard( 'ver', data.ver );
  ShowStr(  'ext', data.ext );
  ShowCard( 'dbg', ORD( data.dbg ) );
  ShowCard( 'vrb', ORD( data.vrb ) );
  ShowStr(  'dir', data.dir );
  ShowCard( 'act', ORD( data.act ) );
  ShowStr(  'arc', data.arc );
END Show;

PROCEDURE InitModule();
BEGIN
  Lib2.Fill( data, 0 );
  data.prg := prg;
  data.ver := ver;
  data.ext := ext;
  data.dbg := FALSE;
  data.arc := 'PAK0.PAK';
  ParseArgs();
  InitDir();
  IF data.vrb THEN Show() END;
END InitModule;

BEGIN
  InitModule();
END Config.
